@@ -1,5 +1,5 @@
 package Selenium::Remote::Driver;
-$Selenium::Remote::Driver::VERSION = '0.1952'; # TRIAL
+$Selenium::Remote::Driver::VERSION = '0.2150'; # TRIAL
 # ABSTRACT: Perl Client for Selenium Remote Driver
 
 use Moo;
@@ -49,6 +49,17 @@ has 'browser_name' => (
     default => sub {'firefox'},
 );
 
+has 'base_url' => (
+    is      => 'rw',
+    lazy    => 1,
+    coerce  => sub {
+        my $base_url = shift;
+        $base_url =~ s|/$||;
+        return $base_url;
+    },
+    predicate => 'has_base_url',
+);
+
 has 'platform' => (
     is      => 'rw',
     coerce  => sub { ( defined($_[0]) ? $_[0] : 'ANY' )},
@@ -74,7 +85,7 @@ has 'webelement_class' => (
 
 has 'default_finder' => (
     is      => 'rw',
-    coerce  => sub { FINDERS->{ $_[0] } },
+    coerce  => sub { __PACKAGE__->FINDERS->{ $_[0] } },
     default => sub {'xpath'},
 );
 
@@ -169,6 +180,26 @@ has 'desired_capabilities' => (
     predicate => 'has_desired_capabilities'
 );
 
+has 'inner_window_size' => (
+    is        => 'rw',
+    lazy      => 1,
+    predicate => 1,
+    coerce    => sub {
+        my $size = shift;
+
+        croak "inner_window_size must have two elements: [ height, width ]"
+          unless scalar @$size == 2;
+
+        foreach my $dim (@$size) {
+            croak 'inner_window_size only accepts integers, not: ' . $dim
+              unless Scalar::Util::looks_like_number($dim);
+        }
+
+        return $size;
+    },
+
+);
+
 has 'testing' => (
     is => 'rw',
     default => sub { 0 },
@@ -191,6 +222,10 @@ sub BUILD {
         if ( !( defined $self->session_id ) ) {
             croak "Could not establish a session with the remote server\n";
         }
+        elsif ($self->has_inner_window_size) {
+            my $size = $self->inner_window_size;
+            $self->set_inner_window_size(@$size);
+        }
     }
 }
 
@@ -217,10 +252,15 @@ sub _execute_command {
     my ( $self, $res, $params ) = @_;
     $res->{'session_id'} = $self->session_id;
     my $resource = $self->commands->get_params($res);
+
     if ($resource) {
-        my $resp =
-          $self->remote_conn->request( $resource->{'method'},
-            $resource->{'url'}, $params );
+        $params = {} unless $params;
+        my $resp = $self->remote_conn->request(
+            $resource->{method},
+            $resource->{url},
+            $resource->{no_content_success},
+            $params
+        );
         if ( ref($resp) eq 'HASH' ) {
             if ( $resp->{cmd_status} && $resp->{cmd_status} eq 'OK' ) {
                 return $resp->{cmd_return};
@@ -272,7 +312,7 @@ sub new_session {
     }
 
     if ($args->{desiredCapabilities}->{browserName} =~ /firefox/i
-        && $self->has_firefox_profile) {
+          && $self->has_firefox_profile) {
         $args->{desiredCapabilities}->{firefox_profile} = $self->firefox_profile;
     }
 
@@ -294,14 +334,17 @@ sub _request_new_session {
     # TODO: rewrite the testing better, this is so fragile.
     my $resp = $self->remote_conn->request(
         $self->commands->get_method('newSession'),
-        $self->commands->get_url('newSession'), $args,
+        $self->commands->get_url('newSession'),
+        $self->commands->get_no_content_success('newSession'),
+        $args,
     );
     if ( ( defined $resp->{'sessionId'} ) && $resp->{'sessionId'} ne '' ) {
         $self->session_id( $resp->{'sessionId'} );
     }
     else {
         my $error = 'Could not create new session';
-        $error .= ": $resp->{cmd_return}" if defined $resp->{cmd_return};
+        $error .= ': ' . $resp->{cmd_return}->{message}
+          if exists $resp->{cmd_return}->{message};
         croak $error;
     }
 }
@@ -494,8 +537,14 @@ sub navigate {
 
 sub get {
     my ( $self, $url ) = @_;
+
+    if ($self->has_base_url && $url !~ m|://|) {
+        $url =~ s|^/||;
+        $url = $self->base_url . "/" . $url;
+    }
+
     my $res    = { 'command' => 'get' };
-    my $params = { 'url'     => $url };
+    my $params = { 'url'     => $url  };
     return $self->_execute_command( $res, $params );
 }
 
@@ -530,7 +579,7 @@ sub refresh {
 
 sub has_javascript {
     my $self = shift;
-    return $self->javascript == JSON::true;
+    return int($self->javascript);
 }
 
 
@@ -687,20 +736,12 @@ sub switch_to_window {
 
 
 sub get_speed {
-    my ($self) = @_;
-    my $res = { 'command' => 'getSpeed' };
-    return $self->_execute_command($res);
+    carp 'get_speed is deprecated and will be removed in the upcoming version of this module';
 }
 
 
 sub set_speed {
-    my ( $self, $speed ) = @_;
-    if ( not defined $speed ) {
-        return 'Speed not provided.';
-    }
-    my $res    = { 'command' => 'setSpeed' };
-    my $params = { 'speed'   => $speed };
-    return $self->_execute_command( $res, $params );
+    carp 'set_speed is deprecated and will be removed in the upcoming version of this module';
 }
 
 
@@ -713,10 +754,7 @@ sub set_window_position {
     my $res = { 'command' => 'setWindowPosition', 'window_handle' => $window };
     my $params = { 'x' => $x, 'y' => $y };
     my $ret = $self->_execute_command( $res, $params );
-    if ( $ret =~ m/204/g ) {
-        return 1;
-    }
-    else { return 0; }
+    return $ret ? 1 : 0;
 }
 
 
@@ -729,10 +767,7 @@ sub set_window_size {
     my $res = { 'command' => 'setWindowSize', 'window_handle' => $window };
     my $params = { 'height' => $height, 'width' => $width };
     my $ret = $self->_execute_command( $res, $params );
-    if ( $ret =~ m/204/g ) {
-        return 1;
-    }
-    else { return 0; }
+    return $ret ? 1 : 0;
 }
 
 
@@ -803,7 +838,7 @@ sub find_element {
         croak 'Search string to find element not provided.';
     }
     my $using =
-      ( defined $method ) ? FINDERS->{$method} : $self->default_finder;
+      ( defined $method ) ? $self->FINDERS->{$method} : $self->default_finder;
     if ( defined $using ) {
         my $res = { 'command' => 'findElement' };
         my $params = { 'using' => $using, 'value' => $query };
@@ -840,7 +875,7 @@ sub find_elements {
     }
 
     my $using =
-      ( defined $method ) ? FINDERS->{$method} : $self->default_finder;
+      ( defined $method ) ? $self->FINDERS->{$method} : $self->default_finder;
 
     if ( defined $using ) {
         my $res = { 'command' => 'findElements' };
@@ -885,9 +920,9 @@ sub find_child_element {
         croak "Missing parameters";
     }
     my $using = ( defined $method ) ? $method : $self->default_finder;
-    if ( exists FINDERS->{$using} ) {
+    if ( exists $self->FINDERS->{$using} ) {
         my $res = { 'command' => 'findChildElement', 'id' => $elem->{id} };
-        my $params = { 'using' => FINDERS->{$using}, 'value' => $query };
+        my $params = { 'using' => $self->FINDERS->{$using}, 'value' => $query };
         my $ret_data = eval { $self->_execute_command( $res, $params ); };
         if ($@) {
             if ( $@
@@ -920,9 +955,9 @@ sub find_child_elements {
         croak "Missing parameters";
     }
     my $using = ( defined $method ) ? $method : $self->default_finder;
-    if ( exists FINDERS->{$using} ) {
+    if ( exists $self->FINDERS->{$using} ) {
         my $res = { 'command' => 'findChildElements', 'id' => $elem->{id} };
-        my $params = { 'using' => FINDERS->{$using}, 'value' => $query };
+        my $params = { 'using' => $self->FINDERS->{$using}, 'value' => $query };
         my $ret_data = eval { $self->_execute_command( $res, $params ); };
         if ($@) {
             if ( $@
@@ -939,7 +974,7 @@ sub find_child_elements {
                 die $@;
             }
         }
-        my $elem_obj_arr;
+        my $elem_obj_arr = [];
         my $i = 0;
         foreach (@$ret_data) {
             $elem_obj_arr->[$i] =
@@ -1047,7 +1082,7 @@ sub upload_file {
     require MIME::Base64;
 
     my $params = {
-        file => encode_base64($string)          # base64-encoded string
+        file => MIME::Base64::encode_base64($string)          # base64-encoded string
     };
     return $self->_execute_command( $res, $params );
 }
@@ -1061,7 +1096,7 @@ sub get_text {
 
 sub get_body {
     my $self = shift;
-    return $self->get_text('//body');
+    return $self->get_text('//body', 'xpath');
 }
 
 
@@ -1075,6 +1110,43 @@ sub get_path {
 }
 
 
+sub set_inner_window_size {
+    my $self = shift;
+    my $height = shift;
+    my $width = shift;
+    my $location = $self->get_current_url;
+
+    $self->execute_script('window.open("' . $location . '", "_blank")');
+    $self->close;
+    my @handles = @{ $self->get_window_handles };
+    $self->switch_to_window(pop @handles);
+
+    my @resize = (
+        'window.innerHeight = ' . $height,
+        'window.innerWidth  = ' . $width,
+        'return 1'
+    );
+
+    return $self->execute_script(join(';', @resize)) ? 1 : 0;
+}
+
+
+sub get_local_storage_item {
+    my ($self, $key) = @_;
+    my $res = { 'command' => 'getLocalStorageItem' };
+    my $params = { 'key' => $key };
+    return $self->_execute_command($res, $params);
+}
+
+
+sub delete_local_storage_item {
+    my ($self, $key) = @_;
+    my $res = { 'command' => 'deleteLocalStorageItem' };
+    my $params = { 'key' => $key };
+    return $self->_execute_command($res, $params);
+}
+
+
 1;
 
 __END__
@@ -1089,7 +1161,7 @@ Selenium::Remote::Driver - Perl Client for Selenium Remote Driver
 
 =head1 VERSION
 
-version 0.1952
+version 0.2150
 
 =head1 SYNOPSIS
 
@@ -1196,11 +1268,12 @@ available here.
     are also optional.
 
         'auto_close'           - <boolean>  - whether driver should end session on remote server on close.
+        'base_url'             - <string>   - OPTIONAL, base url for the website Selenium acts on. This can save you from repeating the domain in every call to $driver->get()
         'default_finder'       - <string>   - choose default finder used for find_element* {class|class_name|css|id|link|link_text|name|partial_link_text|tag_name|xpath}
+        'inner_window_size'    - <aref[Int]>- An array ref [ height, width ] that the browser window should use as its initial size immediately after instantiation
         'webelement_class'     - <string>   - sub-class of Selenium::Remote::WebElement if you wish to use an alternate WebElement class.
         'ua'                   - LWP::UserAgent instance - if you wish to use a specific $ua, like from Test::LWP::UserAgent
 
-
     If no values are provided, then these defaults will be assumed:
         'remote_server_addr' => 'localhost'
         'port'               => '4444'
@@ -1268,12 +1341,12 @@ available here.
  Input:
     The only respected keys in the input hash are:
 
+        desired_capabilities - HASHREF - defaults to {}
         remote_server_addr   - STRING  - defaults to localhost
         port                 - INTEGER - defaults to 4444
         default_finder       - STRING  - defaults to xpath
         webelement_class     - STRING  - defaults to Selenium::Remote::WebElement
         auto_close           - BOOLEAN - defaults to 1
-        desired_capabilities - HASHREF - defaults to {}
 
     Except for C<desired_capabilities>, these keys perform exactly the
     same as listed in the regular "new" constructor.
@@ -1788,9 +1861,19 @@ To conveniently write the screenshot to a file, see L<capture_screenshot()>.
 =head2 switch_to_window
 
  Description:
-    Change focus to another window. The window to change focus to may be
-    specified by its server assigned window handle, or by the value of its name
-    attribute.
+    Change focus to another window. The window to change focus to may
+    be specified by its server assigned window handle, or by the value
+    of the page's window.name attribute.
+
+    If you wish to use the window name as the target, you'll need to
+    have set C<window.name> on the page either in app code or via
+    L</execute_script>, or pass a name as the second argument to the
+    C<window.open()> function when opening the new window. Note that
+    the window name used here has nothing to do with the window title,
+    or the C<< <title> >> element on the page.
+
+    Otherwise, use L</get_window_handles> and select a
+    Webdriver-generated handle from the output of that function.
 
  Input: 1
     Required:
@@ -1808,29 +1891,20 @@ To conveniently write the screenshot to a file, see L<capture_screenshot()>.
 =head2 get_speed
 
  Description:
-    Get the current user input speed. The actual input speed is still browser
-    specific and not covered by the Driver.
-
- Output:
-    STRING - One of these: SLOW, MEDIUM, FAST
-
- Usage:
-    print $driver->get_speed();
+    DEPRECATED - this function is a no-op in Webdriver, and will be
+    removed in the upcoming version of this module. See
+    https://groups.google.com/d/topic/selenium-users/oX0ZnYFPuSA/discussion
+    and
+    http://code.google.com/p/selenium/source/browse/trunk/java/client/src/org/openqa/selenium/WebDriverCommandProcessor.java
 
 =head2 set_speed
 
  Description:
-    Set the user input speed.
-
- Input:
-    STRING - One of these: SLOW, MEDIUM, FAST
-
- Usage:
-    $driver->set_speed('MEDIUM');
-
- Note: This function is a no-op in WebDriver (?). See
-       https://groups.google.com/d/topic/selenium-users/oX0ZnYFPuSA/discussion and
-       http://code.google.com/p/selenium/source/browse/trunk/java/client/src/org/openqa/selenium/WebDriverCommandProcessor.java
+    DEPRECATED - this function is a no-op in Webdriver, and will be
+    removed in the upcoming version of this module. See
+    https://groups.google.com/d/topic/selenium-users/oX0ZnYFPuSA/discussion
+    and
+    http://code.google.com/p/selenium/source/browse/trunk/java/client/src/org/openqa/selenium/WebDriverCommandProcessor.java
 
 =head2 set_window_position
 
@@ -2045,7 +2119,7 @@ To conveniently write the screenshot to a file, see L<capture_screenshot()>.
  Description:
     Send an event to the active element to depress or release a modifier key.
 
-  Input: 2
+ Input: 2
     Required:
       value - String - The modifier key event to be sent. This key must be one 'Ctrl','Shift','Alt',' or 'Command'/'Meta' as defined by the send keys command
       isdown - Boolean/String - Whether to generate a key down or key up
@@ -2159,6 +2233,53 @@ To conveniently write the screenshot to a file, see L<capture_screenshot()>.
  Usage:
      $path = $driver->get_path();
 
+=head2 set_inner_window_size
+
+ Description:
+     Set the inner window size by closing the current window and
+     reopening the current page in a new window. This can be useful
+     when using browsers to mock as mobile devices.
+
+     This sub will be fired automatically if you set the
+     C<inner_window_size> hash key option during instantiation.
+
+ Input:
+     INT - height of the window
+     INT - width of the window
+
+ Output:
+     BOOLEAN - Success or failure
+
+ Usage:
+     $driver->set_inner_window_size(640, 480)
+
+=head2 get_local_storage_item
+
+ Description:
+     Get the value of a local storage item specified by the given key.
+
+ Input: 1
+    Required:
+        STRING - name of the key to be retrieved
+
+ Output:
+     STRING - value of the local storage item
+
+ Usage:
+     $driver->get_local_storage_item('key')
+
+=head2 delete_local_storage_item
+
+ Description:
+     Get the value of a local storage item specified by the given key.
+
+ Input: 1
+    Required
+        STRING - name of the key to be deleted
+
+ Usage:
+     $driver->delete_local_storage_item('key')
+
 =head1 SEE ALSO
 
 Please see those modules/websites for more information related to this module.
@@ -2177,6 +2298,14 @@ L<https://code.google.com/p/selenium/wiki/JsonWireProtocol#Capabilities_JSON_Obj
 
 L<https://github.com/gempesaw/Selenium-Remote-Driver/wiki|https://github.com/gempesaw/Selenium-Remote-Driver/wiki>
 
+=item *
+
+L<Brownie|Brownie>
+
+=item *
+
+L<Wight|Wight>
+
 =back
 
 =head1 BUGS
@@ -2220,6 +2349,10 @@ Allen Lew <allen@alew.org>
 
 =item *
 
+Brian Horakh <brianh@zoovy.com>
+
+=item *
+
 Charles Howes <charles.howes@globalrelay.net>
 
 =item *
@@ -2232,10 +2365,22 @@ Dave Rolsky <autarch@urth.org>
 
 =item *
 
+Dmitry Karasik <dmitry@karasik.eu.org>
+
+=item *
+
 Emmanuel Peroumalnaik <eperoumalnaik@weborama.com>
 
 =item *
 
+Eric Johnson <eric.git@iijo.org>
+
+=item *
+
+Gabor Szabo <gabor@szabgab.com>
+
+=item *
+
 Gordon Child <gordon@gordonchild.com>
 
 =item *
@@ -2248,6 +2393,10 @@ Ivan Kurmanov <duraley@gmail.com>
 
 =item *
 
+Joe Higton <draxil@gmail.com>
+
+=item *
+
 Jon Hermansen <jon.hermansen@gmail.com>
 
 =item *
@@ -2274,6 +2423,14 @@ Tom Hukins <tom@eborcom.com>
 
 Vishwanath Janmanchi <jvishwanath@gmail.com>
 
+=item *
+
+amacleay <a.macleay@gmail.com>
+
+=item *
+
+jamadam <sugama@jamadam.com>
+
 =back
 
 =head1 COPYRIGHT AND LICENSE
