@@ -1,5 +1,5 @@
 package Selenium::Remote::RemoteConnection;
-$Selenium::Remote::RemoteConnection::VERSION = '0.1952'; # TRIAL
+$Selenium::Remote::RemoteConnection::VERSION = '0.2150'; # TRIAL
 #ABSTRACT: Connect to a selenium server
 
 use Moo;
@@ -31,20 +31,27 @@ has 'ua' => (
     builder => sub { return LWP::UserAgent->new; }
 );
 
+has 'error_handler' => (
+    is => 'lazy',
+    builder => sub { return Selenium::Remote::ErrorHandler->new; }
+);
+
 sub BUILD {
     my $self = shift;
     my $status;
     try {
-      $status = $self->request('GET','status');
+        $status = $self->request('GET','status');
     }
     catch {
         croak "Could not connect to SeleniumWebDriver: $_" ;
     };
+
     if($status->{cmd_status} ne 'OK') {
         # Could be grid, see if we can talk to it
         $status = undef;
-        $status = $self->request('GET', 'grid/api/testsession');
+        $status = $self->request('GET', 'grid/api/hub/status');
     }
+
     unless ($status->{cmd_status} eq 'OK') {
         croak "Selenium server did not return proper status";
     }
@@ -53,7 +60,9 @@ sub BUILD {
 
 # This request method is tailored for Selenium RC server
 sub request {
-    my ($self, $method, $url, $params) = @_;
+    my ($self, $method, $url, $no_content_success, $params) = @_;
+    $no_content_success = $no_content_success // 0;
+
     my $content = '';
     my $fullurl = '';
 
@@ -82,7 +91,7 @@ sub request {
         $content = $json->allow_nonref->utf8->encode($params);
     }
 
-    print "REQ: $url, $content\n" if $self->debug;
+    print "REQ: $method, $url, $content\n" if $self->debug;
 
     # HTTP request
     my $header =
@@ -91,11 +100,11 @@ sub request {
     my $request = HTTP::Request->new($method, $fullurl, $header, $content);
     my $response = $self->ua->request($request);
 
-    return $self->_process_response($response);
+    return $self->_process_response($response, $no_content_success);
 }
 
 sub _process_response {
-    my ($self, $response) = @_;
+    my ($self, $response, $no_content_success) = @_;
     my $data; # server response 'value' that'll be returned to the user
     my $json = JSON->new;
 
@@ -105,9 +114,11 @@ sub _process_response {
     else {
         my $decoded_json = undef;
         print "RES: ".$response->decoded_content."\n\n" if $self->debug;
+
         if (($response->message ne 'No Content') && ($response->content ne '')) {
             if ($response->content_type !~ m/json/i) {
-                $data->{'cmd_return'} = 'Server returned error message '.$response->content.' instead of data';
+                $data->{'cmd_status'} = 'NOTOK';
+                $data->{'cmd_return'}->{message} = 'Server returned error message '.$response->content.' instead of data';
                 return $data;
             }
             $decoded_json = $json->allow_nonref(1)->utf8(1)->decode($response->content);
@@ -115,10 +126,9 @@ sub _process_response {
         }
 
         if ($response->is_error) {
-            my $error_handler = Selenium::Remote::ErrorHandler->new;
             $data->{'cmd_status'} = 'NOTOK';
             if (defined $decoded_json) {
-                $data->{'cmd_return'} = $error_handler->process_error($decoded_json);
+                $data->{'cmd_return'} = $self->error_handler->process_error($decoded_json);
             }
             else {
                 $data->{'cmd_return'} = 'Server returned error code '.$response->code.' and no data';
@@ -128,7 +138,12 @@ sub _process_response {
         elsif ($response->is_success) {
             $data->{'cmd_status'} = 'OK';
             if (defined $decoded_json) {
-                $data->{'cmd_return'} = $decoded_json->{'value'};
+                if ($no_content_success) {
+                    $data->{'cmd_return'} = 1
+                }
+                else {
+                    $data->{'cmd_return'} = $decoded_json->{'value'};
+                }
             }
             else {
                 $data->{'cmd_return'} = 'Server returned status code '.$response->code.' but no data';
@@ -159,7 +174,7 @@ Selenium::Remote::RemoteConnection - Connect to a selenium server
 
 =head1 VERSION
 
-version 0.1952
+version 0.2150
 
 =head1 SEE ALSO
 
