@@ -1,5 +1,5 @@
 package Selenium::Remote::WebElement;
-$Selenium::Remote::WebElement::VERSION = '0.1952'; # TRIAL
+$Selenium::Remote::WebElement::VERSION = '0.2150'; # TRIAL
 # ABSTRACT: Representation of an HTML Element used by Selenium Remote Driver
 
 use Moo;
@@ -35,6 +35,7 @@ sub submit {
 
 sub send_keys {
     my ( $self, @strings ) = @_;
+    croak "no keys to send" unless scalar @strings >= 1;
     my $res = { 'command' => 'sendKeysToElement', 'id' => $self->id };
     map { $_ .= "" } @strings;
     my $params = {
@@ -127,6 +128,12 @@ sub is_displayed {
 }
 
 
+sub is_hidden {
+    my ($self) = @_;
+    return ! $self->is_displayed();
+}
+
+
 sub drag {
     my ( $self, $x, $y ) = @_;
     if ( ( not defined $x ) || ( not defined $y ) ) {
@@ -189,7 +196,7 @@ Selenium::Remote::WebElement - Representation of an HTML Element used by Seleniu
 
 =head1 VERSION
 
-version 0.1952
+version 0.2150
 
 =head1 DESCRIPTION
 
@@ -372,6 +379,17 @@ various element related operations can be carried out.
  Usage:
     $elem->is_displayed();
 
+=head2 is_hidden
+
+ Description:
+    Determine if an element is currently hidden.
+
+ Output:
+    BOOLEAN - Whether the element is hidden.
+
+ Usage:
+    $elem->is_hidden();
+
 =head2 drag
 
  Description:
@@ -434,6 +452,9 @@ various element related operations can be carried out.
  Usage:
     $elem->describe();
 
+ Note: DEPRECATED as of 2.42.2 -- use get_text, get_value, is_displayed, or
+ whatever appropriate WebElement function you need instead
+
 =head1 SEE ALSO
 
 Please see those modules/websites for more information related to this module.
